"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const wu_diff_js_1 = require("wu-diff-js");
class UpdateGroup {
    constructor(left, right) {
        this.left = left;
        this.right = right;
    }
    get type() {
        if (!this.left) {
            return "added";
        }
        if (!this.right) {
            return "removed";
        }
        if (this.left.length === this.right.length) {
            return "updated";
        }
        return "replaced";
    }
}
function diff(left, right) {
    const result = wu_diff_js_1.default(left, right);
    return groupDiffResult(result);
}
exports.diff = diff;
function groupDiffResult(result) {
    let l = 0;
    let r = 0;
    let removed = 0;
    let added = 0;
    const groups = [];
    for (const res of result) {
        if (res.type === "added") {
            added++;
            r++;
        }
        else if (res.type === "removed") {
            removed++;
            l++;
        }
        else {
            addGroupIfNeeded();
            added = 0;
            removed = 0;
            l++;
            r++;
        }
    }
    addGroupIfNeeded();
    return groups;
    function addGroupIfNeeded() {
        let left;
        let right;
        if (removed) {
            left = { min: l - removed, length: removed };
        }
        if (added) {
            right = { min: r - added, length: added };
        }
        if (left || right) {
            groups.push(new UpdateGroup(left, right));
        }
    }
}
//# sourceMappingURL=diff.js.map