"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator = require("./generator");
const files_ = require("./files");
const Path = require("path");
const types_1 = require("./types");
const logger_1 = require("./logger");
function run(left, right, options) {
    return __awaiter(this, void 0, void 0, function* () {
        options = Object.assign({ recursive: false, output: null, outdir: null, padding: 20, clusters: 4, css: Path.resolve(__dirname, `../../assets/style.css`), shiftAware: false, threshold: 0, ignoreSpacing: true, verbose: false }, options);
        const logger = new logger_1.Logger(options.verbose);
        logger.verbose("options:");
        logger.verbose("  recursive      :", options.recursive);
        logger.verbose("  output         :", options.output);
        logger.verbose("  outdir         :", options.outdir);
        logger.verbose("  padding        :", options.padding);
        logger.verbose("  clusters       :", options.clusters);
        logger.verbose("  css            :", options.css);
        logger.verbose("  shift-aware    :", options.shiftAware);
        logger.verbose("  threshold      :", options.threshold);
        logger.verbose("  ignore-spacing :", options.ignoreSpacing);
        logger.verbose("  verbose        :", options.verbose);
        const filePairs = options.recursive
            ? files_.getFilePairsRecursively(left, right)
            : files_.getFilePairs(left, right);
        const fileDiffs = {};
        for (let file in filePairs) {
            const filePair = filePairs[file];
            const fileDiff = yield files_.compareFile(filePair, options, logger);
            fileDiffs[file] = fileDiff;
        }
        const leftBaseDir = options.recursive ? left : Path.resolve(".");
        const rightBaseDir = options.recursive ? right : Path.resolve(".");
        generator.generate(fileDiffs, new types_1.Paths(options.css, options.outdir, options.output, leftBaseDir, rightBaseDir));
    });
}
exports.run = run;
//# sourceMappingURL=index.js.map