export declare type FilePairs = {
    [s: string]: FilePair;
};
export declare type FileDiffs = {
    [s: string]: FileDiff;
};
export interface Options {
    recursive?: boolean;
    output?: string;
    outdir?: string;
    padding?: number;
    clusters?: number;
    css?: string;
    shiftAware?: boolean;
    threshold?: number;
    ignoreSpacing?: boolean;
    verbose?: boolean;
}
export declare class Paths {
    srcCss: string;
    outDir: string;
    outHtml: string;
    leftBaseDir: string;
    rightBaseDir: string;
    outCss: string;
    constructor(srcCss: string, outDir: string, outHtml: string, leftBaseDir: string, rightBaseDir: string);
}
export interface FilePair {
    left?: string;
    right?: string;
}
export declare type FileDiffType = "removed" | "added" | "updated" | "unchanged";
export declare class FileDiff {
    left: ImageMetaInfo;
    right: ImageMetaInfo;
    rects: RectsLR;
    constructor(left: ImageMetaInfo, right: ImageMetaInfo, rects: RectsLR);
    readonly type: FileDiffType;
}
export declare type RectsLR = {
    left: Rect[];
    right: Rect[];
};
export declare class Rect {
    left: number;
    top: number;
    right: number;
    bottom: number;
    constructor(left: number, top: number, right: number, bottom: number);
    readonly width: number;
    readonly height: number;
    shift(dx: number, dy: number): Rect;
}
export declare type Point = number[];
export declare type Area = {
    x: number;
    y: number;
    width: number;
    height: number;
};
export declare type DiffResultGroup = {
    type: "points";
    dx: number;
    dy: number;
    points: Point[];
} | {
    type: "area";
    left: Area;
    right: Area;
};
export interface ImageMetaInfo {
    path: string;
    width: number;
    height: number;
}
export interface ImageChange {
    left: ImageMetaInfo;
    right: ImageMetaInfo;
    results: DiffResultGroup[];
}
export interface Image {
    path: string;
    width: number;
    height: number;
    save(path: string): Promise<void>;
    getPixel(x: number, y: number): number[];
    setPixel(x: number, y: number, r: number, g: number, b: number): void;
}
